#!/usr/bin/python

import urllib, urllib2, urlparse, re
import cookielib
from xml.dom import minidom

class APIv3:
	# Class constants / default values.
	VERSION = 3
	DEFAULT_LIMIT = 10
	DEFAULT_OFFSET = 0
	LANG_SV = 'sv'
	LANG_EN = 'en'
	PAYMENT_ACCOUNT = 'account'
	PAYMENT_SMS = 'sms'
	
	#TODO: Strip out default key before shipping.
	def __init__ (self, host = 'http://api.headweb.com', key = '86cc9ddac31d11dea3d733b0e449c0be'):
		"""Default constructor, takes API host server and API key."""
		self.host = host
		self.apiKey = key
		self.loginCompleted = False
		
		# Set up a custom URL opener for urllib2 when used in this context
		# to make sure that we always gets proper cookie handling.
		self.cookieJar = cookielib.LWPCookieJar()
		urllib2.install_opener(urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookieJar)))
	
	def list (self):
		"""EWhr? Lists stuff?"""
		requestString = self._buildBaseString() + '/content?' + urllib.urlencode({'apikey': self.apiKey})
		requestString += self._buildParameterString(parameters);
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		
		return self._parseResponse(data)
	
	def search (self, searchString, **keywordArguments):
		"""Performs a search and matches the supplied string against
		keywords, plot and other elements of each content node."""
		
		requestString = self._buildBaseString() + '/search/' + urllib.quote(searchString) + '?' + urllib.urlencode({'apikey': self.apiKey})
		requestString += self._buildParameterString(keywordArguments)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		return self._parseResponse(data)
	
	def makeStreamPath (self, streamID, authmode, **keywordParameters):
		requestString = self._buildBaseString() + '/player/plex/' + str(streamID) + '?' + urllib.urlencode({'apikey': self.apiKey, 'authmode': authmode})
		requestString += urllib.urlencode(keywordParameters)
		return requestString
	
	def genre (self, genreID, **keywordParameters):
		"""Returns content belonging to a specific genre."""
		if genreID == None:
			requestString = self._buildBaseString() + '/genre'
		else:
			requestString = self._buildBaseString() + '/genre/' + str(genreID)
		requestString += '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		
		return self._parseResponse(data)
	
	def keyword (self, keywordID, **keywordParameters):
		if keywordID == None:
			requestString = self._buildBaseString() + '/keyword'
		else:
			requestString = self._buildBaseString() + '/keyword/' + str(keywordID)
	 	requestString += '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		
		return self._parseResponse(data)
	
	def directory (self, directoryID, **keywordParameters):
		if directoryID == None:
			requestString = self._buildBaseString() + '/directory'
		else:
			requestString = self._buildBaseString() + '/directory/' + str(directoryID)
	 	requestString += '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		
		return self._parseResponse(data)
	
	def videocliptype (self, clipID, **keywordParameters):
		if clipID == None:
			requestString = self._buildBaseString() + '/videocliptype'
		else:
			requestString = self._buildBaseString() + '/videocliptype/' + str(clipID)
	 	requestString += '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		
		return self._parseResponse(data)
	
	def content (self, id, **keywordParameters):
		"""Returns specific content specified by ID."""
		requestString = self._buildBaseString() + '/content/' + str(id) + '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		data = response.read()
		response.close()
		#return self._parseResponse(data)
		return data

	def login (self, username, password, **keywordParameters):
		"""Logs the user into the server over HTTPS. Sets the cookie used in subsequent calls for identification."""
		requestString = self._buildBaseString(True) + '/user/login'
		requestData = urllib.urlencode({'apikey': self.apiKey, 'username': username, 'password': password}) + '&' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString, requestData)
		response = urllib2.urlopen(request)
		responseData = response.read()
		response.close()
		
		result = self._parseResponse(responseData)
		self.loginCompleted = True
		
		return result
	
	def user (self, **keywordParameters):
		"""Returns information about the currently logged in user. Uses the cookie to identify the user."""
		# Send the request to the server to retrieve the XML response.
		requestString = self._buildBaseString() + '/user' + '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		responseData = response.read()
		response.close()
		
		# Create parser and check if an error occured.
		parser = self._parseResponse(responseData)
		
		return parser
	
	def rentals (self, **keywordParameters):
		"""Returns a list of active rentals for the logged in user. Uses the cookie to identify the user."""
		requestString = self._buildBaseString() + '/user/rentals' + '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		responseData = response.read()
		response.close()
		
		return self._parseResponse(responseData)
	
	def encode (self, data):
		return urllib.quote(data)
	
	def _buildBaseString (self, secure = False):
		"""Helper method to build base of URL string. If the 'secure' parameter is True then the URL will be the https equivalent (used for login credentials)."""
		# If the user requests a secure host make sure that we use https.
		if secure == True:
				newHost = re.sub('^http:\/\/', 'https://', self.host, 1)
				return newHost + '/v' + str(self.__class__.VERSION)
		else:
			return self.host + '/v' + str(self.__class__.VERSION)
	
	def _buildParameterString (self, parameters):
		parameterString = ''
		for name, value in parameters.iteritems():
			parameterString += '&%s=%s' % (name, value)
		
		return parameterString
	
	def _parseResponse (self, response):
		"""Creates an XML parser instance and performs basic error checking
		before passing the entire parser object back to the callee."""
		parser = minidom.parseString(response)
		if parser.hasChildNodes() and parser.firstChild.tagName == 'response':
			temp = parser.firstChild
			if temp.hasChildNodes():
				if temp.firstChild.tagName == 'error':
					raise Exception('Server returned error: "%s"' % temp.firstChild.firstChild.nodeValue)
			else:
				raise Exception('DOM "response" element has no child nodes.')
		else:
			raise Exception('Server XML has no root "response" element.')
		return parser
	
	def purchase (self, payment, item, total, redemptionCode = None, \
			**keywordParameters):
		
		# Check 'payment' parameter for valid values.
		if not payment in (Headweb.PAYMENT_ACCOUNT, headweb.PAYMENT_SMS):
			raise Exception('Invalid value for payment parameter("' \
					+ payment + '").')
		
		# Redemption codes only work in account payment mode.
		if redemptionCode != None and payment != Headweb.PAYMENT_ACCOUNT:
			raise Exception('Remdemption codes requires account payment mode.')
		
		# Account payment requires login prior to purchase.
		if payment == Headweb.PAYMENT_ACCOUNT and not self._checkLogin():
			raise Exception('Account payment requested without login.')

		requestString = self._buildBaseString() + '/purchase/0' + '?' + urllib.urlencode({'apikey': self.apiKey}) + '&' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		responseData = response.read()
		response.close()
		
		return self._parseResponse(responseData)

	def activate (self, authcode, **keywordParameters):
		"""Activates an SMS purchase initiated via a call to "purchase"."""
		cleanedCode = authcode.upper()
		requestString = self._buildBaseString() + '/purchase/activate' + '?' + urllib.urlencode({'apikey': self.apiKey, 'authcode': cleanedCode}) + '=' + urllib.urlencode(keywordParameters)
		request = urllib2.Request(requestString)
		response = urllib2.urlopen(request)
		responseData = response.read()
		response.close()
		
		return self._parseResponse(responseData)
	
	def _checkLogin (self):
		"""Local function to check weather cookies are set for login."""
		if len(self.cookieJar) > 0 and self.loginCompleted:
			return True
		else:
			return False
