import Headweb
from datetime import datetime
from xml.dom import minidom
from copy import deepcopy

class Amount (object):
	def __init__ (self, raw = 0.0, full = '0.00 SEK', humanReadable = '0 kr'):
		self.raw = raw
		self.full = full
		self.humanReadable = humanReadable
	
	def parseDom (self, dom):
		if dom.nodeType != minidom.Node.ELEMENT_NODE:
			raise Exception('DOM node is not an element type node.')
		
		if dom.hasAttribute('raw'):
			self.raw = dom.getAttribute('raw')
		else:
			raise Exception('"Amount" entity has no "raw" attribute.')
		
		if dom.hasAttribute('full'):
			self.full = dom.getAttribute('full')
		else:
			raise Exception('"Amount" entity has no "full" attribute.')
		
		if dom.hasChildNodes() and dom.firstChild.nodeType == \
				minidom.Node.TEXT_NODE:
			self.humanReadable = dom.firstChild.nodeValue
		else:
			raise Exception('"Amount" entity has no child text node.')
	
	def __str__ (self):
		return self.humanReadable
	
class Image (object):
	"""Represents a server-side image object with URL and dimensions."""
	def __init__ (self, url = None, width = None, height = None):
		self.url = None
		self.width = None
		self.height = None
	
	def parseDom (self, node):
		# Process height and width attributes.
		if node.hasAttribute('width'):
			self.width = int(node.getAttribute('width'))
		
		if node.hasAttribute('height'):
			self.height = int(node.getAttribute('height'))
		
		# Text node contents (image URL).
		if node.hasChildNodes() and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
			self.url = node.firstChild.nodeValue
		else:
			raise Exception('Unable to find a text child node (with URL) for image.')



class Stream (object):
	TYPE_FLASH = 'flash'
	TYPE_SILVERLIGHT = 'silverlight'
	STATUS_PUB = 'pub'
	STATUS_SOFTPUB = 'softpub'
	STATUS_PUBWAIT = 'pubwait'
	
	def __init__ (self, id = None, type = None, status = None, pubdate = None, \
			runtime = None, price = None, preview = None):
		self.id = id
		self.type = type
		self.status = status
		self.publishingDate = pubdate
		self.runtime = runtime
		self.price = price
		self.preview = preview

	def parseDom (self, dom):
		if dom.tagName != 'stream':
			raise Exception('DOM node is not an "stream" element.')
		
		if dom.hasAttribute('id'):
			self.id = int(dom.getAttribute('id'))
		else:
			raise Exception('DOM node has no "id" attribute.')
		
		for child in dom.childNodes:
			if child.tagName == 'price':
				amount = Amount()
				amount.parseDom(child)
				self.price = amount
			
			elif child.tagName == 'status' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				status = child.firstChild.nodeValue
				
				if not status in (Stream.STATUS_PUB, Stream.STATUS_SOFTPUB, Stream.STATUS_PUBWAIT):
					raise Exception('Unknown value for "status" element (' + status + ').')
				self.status = status
			
			elif child.tagName == 'type':
				type = child.firstChild.nodeValue
				if not type in (Stream.TYPE_FLASH, Stream.TYPE_SILVERLIGHT):
					raise Exception('Unknown value for "type" element.')
				self.type = type
			
			elif child.tagName == 'live':
				live = child.firstChild.nodeValue
				if live == 'true':
					self.live = True
				elif live == 'false':
					self.live = False
				else:
					raise Exception('Unknown value for "live" element.')
			
			elif child.tagName == 'pubdate':
				self.publishingDate = child.getAttribute('utc')
			
			elif child.tagName == 'runtime':
				self.runtime = int(child.firstChild.nodeValue)
			
			elif child.tagName == 'preview' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.preview = int(child.firstChild.nodeValue)
		
class SMSPayment (object):
	def __init__ (self):
		self.amount = None
		self.code = None
		self.number = None
	
	def parseDom (self, dom):
		for child in dom.childNodes:
			if child.tagName == 'amount':
				amount = Amount()
				amount.parseDom(child)
				self.amount = amount
			
			elif child.tagName == 'code' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.code = child.firstChild.nodeValue
			
			elif child.tagName == 'number' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.number = child.firstChild.nodeValue
			
class PurchaseStatus (object):
	def __init__ (self):
		self.success = None
		self.status = None
		self.sms = None
		self.items = None
	
	def parseDom (self, dom):
		
		for child in dom.childNodes:
			if child.tagName == 'status' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				status = child.firstChild.nodeValue
				if not status in ('captured', 'authfailed'):
					raise Exception('Invalid value for "status" element.')
			
			elif child.tagName == 'sms' and child.hasChildNodes():
				payment = SMSPayment()
				payment.parseDom(child)
				sms = payment
		
class UserInfo (object):
	def __init__ (self, userName = None, firstName = None,
			lastName = None, id = None, balance = Amount(), credits = None):
		self.userName = userName
		self.firstName = firstName
		self.lastName = lastName
		self.id = id
		self.balance = balance
		self.credits = credits
	
	def parseDom (self, dom):
		if dom.tagName != 'user':
			raise Exception('DOM node is not an "user" element.')
		
		if dom.hasAttribute('id'):
			self.id = int(dom.getAttribute('id'))
		else:
			raise Exception('DOM node has no "id" attribute.')
		
		for child in dom.childNodes:
			if child.tagName == 'username' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.userName = child.firstChild.nodeValue
			
			elif child.tagName == 'firstname' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.firstName = child.firstChild.nodeValue
			
			elif child.tagName == 'lastname' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.lastName = child.firstChild.nodeValue
			
			elif child.tagName == 'balance':
				amount = Amount()
				amount.parseDom(child)
				self.balance = amount
			
			elif child.tagName == 'credits' and child.hasChildNodes and \
					child.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.credits = child.firstChild.nodeValue
			
class Genre (object):
	def __init__ (self, name = None, id = None):
		self.name = name
		self.id = id
	
	def parseDom (self, dom):
		if dom.tagName == 'genre':
			if dom.hasAttribute('id'):
				self.id = int(dom.getAttribute('id'))
			else:
				raise Exception('"genre" node is missing "id" attribute.')
			
			if dom.hasChildNodes() and dom.firstChild.nodeType == \
					minidom.Node.TEXT_NODE:
				self.name = dom.firstChild.nodeValue
			else:
				raise Exception('"genre" node has no child text node for name.')
		else:
			raise Exception('DOM node is not a "genre" element.')
	
class Keyword (object):
	def __init__ (self, name = None, id = None):
		self.name = name
		self.id = id
	
	def parseDom (self, dom):
		if dom.tagName == 'keyword':
			if dom.hasAttribute('id'):
				self.id = int(dom.getAttribute('id'))
			else:
				raise Exception('"keyword" node is missing "id" attribute.')
			
			if dom.hasChildNodes() and dom.firstChild.nodeType == \
					minidom.Node.TEXT_NODE:
				self.name = dom.firstChild.nodeValue
			else:
				raise Exception('"keyword" node has no child text node for name.')
		else:
			raise Exception('DOM node is not a "keyword" element.')

class Directory (object):
	def __init__ (self, id = None, name = None):
		self.id = id
		self.name = name
	
	def parseDom (self, dom):
		if dom.tagName == 'directory':
			if dom.hasAttribute('id'):
				self.id = int(dom.getAttribute('id'))
			else:
				raise Exception('"directory" node is missing "id" attribute.')
			
			if dom.hasChildNodes() and dom.firstChild.nodeType == \
					minidom.Node.TEXT_NODE:
				self.name = dom.firstChild.nodeValue
			else:
				raise Exception('"directory" node has no child text node for name.')
		else:
			raise Exception('DOM node is not a "directory" element.')

class VideoClipType (object):
	def __init__ (self, id = None, name = None):
		self.id = id
		self.name = name
	
	def parseDom (self, dom):
		if dom.tagName == 'videocliptype':
			if dom.hasAttribute('id'):
				self.id = int(dom.getAttribute('id'))
			else:
				raise Exception('"videocliptype" node is missing "id" attribute.')
			
			if dom.hasChildNodes() and dom.firstChild.nodeType == \
					minidom.Node.TEXT_NODE:
				self.name = dom.firstChild.nodeValue
			else:
				raise Exception('"videocliptype" node has no child text node for name.')
		else:
			raise Exception('DOM node is not a "videocliptype" element.')

class VideoClip (object):
	def __init__ (self, id = None, name = None, pubdate = None, \
			width = None, height = None, url = None):
		self.id = id
		self.name = name
		self.publishingDate = pubdate
		self.width = width
		self.height = height
		self.url = url
	
	def parseDom (self, dom):
		if dom.tagName != 'videoclip':
			raise Exception('DOM node is not a "videoclip" element.')
		
		if dom.hasAttribute('id'):
			self.id = int(dom.getAttribute('id'))
		else:
			raise Exception('"videoclip" node is missing "id" attribute.')
			
		for node in dom.childNodes:
			# Name of movie/title/item.
			if node.tagName == 'name' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.name = node.firstChild.nodeValue
			
			# Publishing date at Headweb.
			elif node.tagName == 'pubdate':
				self.publishingDate = datetime.fromtimestamp(int(node.getAttribute('utc')))
			
			# Video width
			elif node.tagName == 'width' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.width = int(node.firstChild.nodeValue)
			
			# Video height
			elif node.tagName == 'height' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.height = int(node.firstChild.nodeValue)
			
			# Video url
			elif node.tagName == 'url' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.url = node.firstChild.nodeValue

class RentalItem (object):
	"""Represents a purchased item."""
	def __init__ (self, id = None, purchaseId = None, name = None, \
			purchased = None, status = None, timeleft = None, auth = None):
		self.id = id
		self.purchaseId = purchaseId
		self.name = name
		self.purchased = purchased
		self.status = status
		self.timeleft = timeleft
		self.auth = auth
	
	def parseDom (self, dom):
		if not dom.tagName == 'item':
			raise Exception('DOM node is not an "item" element.')
			
		# Extract the ID of the title/entity itself.
		if dom.hasAttribute('id'):
			self.id = int(dom.getAttribute('id'))
		else:
			raise Exception('DOM "item" element is missing "id" attribute.')
		
		# Extract the purchase ID.
		if dom.hasAttribute('purchid'):
			self.purchaseId = int(dom.getAttribute('purchid'))
		else:
			raise Exception('DOM "item" element is missing "purchid" attribute.')
		for node in dom.childNodes:
			# Name of movie/title/item.
			if node.tagName == 'name' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.name = node.firstChild.nodeValue
				
			# State of item (available etc).
			elif node.tagName == 'state' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				state = node.firstChild.nodeValue
				if not state in ('expired', 'timeleft', 'available'):
					raise Exception('Unrecognized value for "state" element.')
			
			# Time left for rental.
			elif node.tagName == 'timeleft' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.timeleft = int(node.firstChild.nodeValue)
			
			# Date when the item was purchased.
			elif node.tagName == 'purchased':
				if node.hasAttribute('utc'):
					self.purchased = datetime.fromtimestamp(node.getAttribute('utc'))
			
class Item (object):
	"""Represents an item from the web API, most of the time a movie."""
	def __init__ (self):
		self.id = None
		self.year = None
		self.plot = None
		self.name = None
		self.originalName = None
		self.url = None
		self.stream = None
		self.cover = []
		self.genre = None
	
	def parseDom (self, dom):
		# If a minidom node was supplied, assume it is an 'item' node.
		if not dom.tagName == 'content':
			raise Exception('DOM node is not an "content" element ("' + \
					dom.tagName + '").')
			
		# Extract the ID of the title/entity itself.
		if not dom.hasAttribute('id'):
			raise Exception('DOM "item" element is missing "id" attribute.')
		self.id = int(dom.getAttribute('id'))
		
		for node in dom.childNodes:
			# Name of movie/title/item.
			if node.tagName == 'name' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.name = node.firstChild.nodeValue
			
			# Check weather we have an original name (optional).
			elif node.tagName == 'originalname' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.name = node.firstChild.nodeValue
			
			elif node.tagName == 'genre' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.genre = node.firstChild.nodeValue
			
			elif node.tagName == 'year' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.year = int(node.firstChild.nodeValue)
			
			elif node.tagName == 'plot' and node.hasChildNodes() \
					and node.firstChild.nodeType == minidom.Node.TEXT_NODE:
				self.plot = node.firstChild.nodeValue
			
			# Stream entities
			elif node.tagName == 'stream':
				stream = Stream()
				stream.parseDom(node)
				self.stream = stream
			
			# Cover images (multiple image resolutions).
			# Example: <cover width="100" height="160">http://foobar.com/image.jpg</cover>
			elif node.tagName == 'cover' and node.hasChildNodes():
				image = Image()
				image.parseDom(node)
				self.cover.append(image)

		# Once parsed all the cover images should be sorted by size (rising).
		self.cover.sort(key = lambda s: s.width * s.height)
	
	def getCover (self, quality):
		"""Returns the cover image object matching the quality parameter.
		The quality parameter is a real number (float) between 0.0 and 1.0."""
		
		if quality < 0.0 or quality > 1.0:
			raise ValueError('Quality parameter not in range 0.0 to 1.0.')
		
		index = int(round(quality * (len(self.cover) - 1)))
		
		return self.cover[index]
	
class Query:
	# The fields list must be sorted alphabetically for performance on server.
	DEFAULT_FIELDS = 'comment,cover,genre,name,plot,pubdate,rating,related,stream,year'
	DEFAULT_LIMIT = 10
	DEFAULT_VERSION = 3
	
	def __init__ (self, api, limit = DEFAULT_LIMIT):
		self.api = api
		self.version = Query.DEFAULT_VERSION
		self.parameters = None
		self.items = 0
		self.limit = limit
		self.offset = 0
		self.list = []
	
	def _checkParameters (self, **arguments):
		"""Checks that arguments doesn't contain 'limit', 'offset' or 'fields'.
		These are reserved for the paging control mechanism."""
		keys = arguments.keys()
		if 'limit' in keys:
			raise Exception('Named parameter "limit" is not allowed.')
		elif 'offset' in keys:
			raise Exception('Named parameter "offset" is not allowed.')
		elif 'fields' in keys:
			raise Exception('Named parameter "fields" is not allowed.')
	
	def _checkInitialParameters (self, **arguments):
		"""Checks that arguments doesn't contain 'limit', 'offset' or 'fields'.
		These are reserved for the paging control mechanism."""
		keys = arguments.keys()
		if 'limit' in keys:
			raise Exception('Named parameter "limit" is not allowed.')
		elif 'offset' in keys:
			raise Exception('Named parameter "offset" is not allowed.')
		elif 'fields' in keys:
			raise Exception('Named parameter "fields" is not allowed.')
	
	@staticmethod
	def _mergeParameters (existingParameters, **newParameters):
		"""Merges a set of keyword parameters with an existing bunch.
		Duplicate keys is an error and throws an exception."""
		
		mergedParameters = deepcopy(existingParameters)
		
		for key, value in newParameters.iteritems():
			if key in existingParameters:
				raise Exception('Error: Duplicate parameter "%s" detected in query.' % key)
			mergedParameters[key] = value
		
		return mergedParameters
	
	def hasNext (self):
		if self.items - self.offset > 0:
			return True
		else:
			return False
	
	def hasPrev (self):
		if self.offset > 0:
			return True
		else:
			return False
	
	def pageNext (self):
		if self.hasNext():
			self.offset = self.offset + self.limit
			# Clamp offset to maximum value (integer multiple of limit).
			if self.offset >= self.items:
				self.offset = int(self.items / self.limit)
			return True
		else:
			return False
			
	def pagePrev (self):
		if self.hasPrev():
			self.offset = self.offset - self.limit
			# Clamp offset to minimum value.
			if self.offset < 0:
				self.offset = 0
			return True
		else:
			return False

	def _parseCommon (self, parser):
		if parser.hasChildNodes() and parser.firstChild.tagName == 'response':
			
			if parser.firstChild.hasAttribute('lang'):
				self.language = parser.firstChild.getAttribute('lang')
			
			# If a list is returned then parse the size attributes.
			if parser.firstChild.hasChildNodes():
				listFound = False
				for child in parser.firstChild.childNodes:
					if child.tagName == 'list':
						
						if child.hasAttribute('items'):
							self.items = int(child.getAttribute('items'))
						if child.hasAttribute('limit'):
							self.limit = int(child.getAttribute('limit'))
						if child.hasAttribute('offset'):
							self.offset = int(child.getAttribute('offset'))
						
						listFound = True
						break;
						
				if not listFound:
					raise Exception('Server XML "response" element has no "list" child element.')
			else:
				raise Exception('Server XML "response" element has no child elements.')
				
		else:
			raise Exception('Server XML has no "response" element.')
	
	def __iter__ (self):
		return self.list.__iter__()
	
	def __next__ (self):
		return self.list.__next__()
	
	def getItemById (self, id):
		"""Return an item in the query specified by ID.b Returns None if no
		matching ID could be found."""
		for item in self.list:
			if self.id == id:
				return item
		return None
	
class SearchQuery (Query):
	def __init__ (self, api, searchString, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		Query._checkInitialParameters(self, **namedParameters)
		Query.__init__(self, api, limit)
		self.api = api
		self.searchString = searchString
		self.parameters = namedParameters
		self._query()
	
	def _query (self):
		temporaryParameters = Query._mergeParameters(self.parameters, \
				fields=Query.DEFAULT_FIELDS, offset=self.offset, \
				limit=self.limit)
		parser = self.api.search(self.searchString, **temporaryParameters)
		Query._parseCommon(self, parser)
		self.list = []
		
		for item in parser.firstChild.firstChild.childNodes:
			newItem = Item()
			newItem.parseDom(item)
			self.list.append(newItem)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result
	
class KeywordQuery (Query):
	def __init__ (self, api, keyword = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		Query._checkInitialParameters(self, **namedParameters)
		Query.__init__(self, api, limit)
		self.keyword = keyword
		self.parameters = namedParameters
		self._query()
		
	def _query (self):
		temporaryParameters = Query._mergeParameters(self.parameters, \
				fields=Query.DEFAULT_FIELDS, offset=self.offset, \
				limit=self.limit)
		parser = self.api.keyword(self.keyword, **temporaryParameters)
		Query._parseCommon(self, parser)
		self.list = []
		
		# If we used None as the genre then we get a list of Keyword items.
		if self.keyword == None:
			for keyword in parser.firstChild.firstChild.childNodes:
				keywordItem = Keyword()
				keywordItem.parseDom(keyword)
				self.list.append(keywordItem)
		
		# Otherwise we queried for titles for a specific keyword.
		else:
			for item in parser.firstChild.firstChild.childNodes:
				newItem = Item()
				newItem.parseDom(item)
				self.list.append(newItem)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result

class GenreQuery (Query):
	def __init__ (self, api, genre = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		Query._checkInitialParameters(self, **namedParameters)
		Query.__init__(self, api, limit)
		self.genre = genre
		self.parameters = namedParameters
		self._query()
	
	def _query (self):
		temporaryParameters = Query._mergeParameters(self.parameters, \
				fields=Query.DEFAULT_FIELDS, offset=self.offset, \
				limit=self.limit)
		parser = self.api.genre(self.genre, **temporaryParameters)
		Query._parseCommon(self, parser)
		self.list = []
		
		# If we used None as the genre then we get a list of Genre items.
		if self.genre == None:
			for genre in parser.firstChild.firstChild.childNodes:
				genreItem = Genre()
				genreItem.parseDom(genre)
				self.list.append(genreItem)
		
		# Otherwise we queried for titles for a specific genre.
		else:
			for item in parser.firstChild.firstChild.childNodes:
				newItem = Item()
				newItem.parseDom(item)
				self.list.append(newItem)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result

class DirectoryQuery (Query):
	def __init__ (self, api, directory = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		Query.__init__(self, api, limit)
		Query._checkInitialParameters(self, **namedParameters)
		self.directory = directory
		self.parameters = namedParameters
		self._query()
	
	def _query (self):
		temporaryParameters = Query._mergeParameters(self.parameters, \
				fields=Query.DEFAULT_FIELDS, offset=self.offset, \
				limit=self.limit)
		parser = self.api.directory(self.directory, **temporaryParameters)
		Query._parseCommon(self, parser)
		self.list = []
		
		# If we used None, then we get a list of Directory items.
		if self.directory == None:
			for directory in parser.firstChild.firstChild.childNodes:
				directoryItem = Directory()
				directoryItem.parseDom(directory)
				self.list.append(directoryItem)
		
		# Otherwise we queried for titles for a specific directory.
		else:
			for item in parser.firstChild.firstChild.childNodes:
				newItem = Item()
				newItem.parseDom(item)
				self.list.append(newItem)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result

class VideoClipTypeQuery (Query):
	def __init__ (self, api, videoClipType = None, limit = Query.DEFAULT_LIMIT,\
			**namedParameters):
		Query._checkInitialParameters(self, **namedParameters)
		Query.__init__(self, api, limit)
		self.videoClipType = videoClipType
		self.parameters = namedParameters
		self._query()
	
	def _query (self):
		temporaryParameters = Query._mergeParameters(self.parameters, \
				fields=Query.DEFAULT_FIELDS, offset=self.offset, \
				limit=self.limit)
		parser = self.api.videocliptype(self.videoClipType, **temporaryParameters)
		Query._parseCommon(self, parser)
		self.list = []
		
		# If we used None, then we get a list of VideoClipType items.
		if self.videoClipType == None:
			for clipType in parser.firstChild.firstChild.childNodes:
				videoClipType = VideoClipType()
				videoClipType.parseDom(clipType)
				self.list.append(videoClipType)
		
		# Otherwise we queried for VideoClip instances for a specific type.
		else:
			for clip in parser.firstChild.firstChild.childNodes:
				videoClip = VideoClip()
				videoClip.parseDom(clip)
				self.list.append(videoClip)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result

class RentalsQuery (Query):
	def __init__ (self, api, limit = Query.DEFAULT_LIMIT, **namedParameters):
		Query._checkInitialParameters(self, **namedParameters)
		Query.__init__(self, api, limit)
		self.parameters = namedParameters
		parser = api.rentals(**namedParameters)

	def _query (self):
		parser = self.api.rentals(**self.parameters)
		Query._parseCommon(self, parser)
		self.list = []
		for child in parser.firstChild.firstChild.childNodes:
			rental = RentalItem()
			rental.parseDom(child)
			self.list.append(rental)
	
	def submit (self):
		self._query()
	
	def pageNext (self):
		result = Query.pageNext(self)
		if result:
			self._query()
		return result
	
	def pagePrev (self):
		result = Query.pagePrev(self)
		if result:
			self._query()
		return result

class APIv3:
	def __init__ (self, host = 'http://api.headweb.com', key = '86cc9ddac31d11dea3d733b0e449c0be'):
		self.api = Headweb.APIv3(host, key)
	
	def login (self, username, password, **namedParameters):
		userInfo = UserInfo()
		parser = self.api.login(username, password, **namedParameters)
		userInfo.parseDom(parser.firstChild.firstChild)
		return userInfo
	
	def search (self, searchString, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		query = SearchQuery(self.api, searchString, limit, **namedParameters)
		return query
	
	def keyword (self, keywordId = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		return KeywordQuery(self.api, keywordId, **namedParameters)
	
	def genre (self, genreId = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		return GenreQuery(self.api, genreId, **namedParameters)
	
	def directory (self, directoryId = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		return DirectoryQuery(self.api, directoryId, **namedParameters)
	
	def videocliptype (self, clipId = None, limit = Query.DEFAULT_LIMIT, \
			**namedParameters):
		return VideoClipTypeQuery(self.api, clipId, **namedParameters)
	
	def purchase (self, payment, items, total, redemptionCode = None, \
			**namedParameters):
		if payment == Headweb.PAYMENT_ACCOUNT:
			purchase = AccountPayment(self.api)
		elif payment == Headweb.PAYMENT_SMS:
			purchase = SMSPayment(self.api)
		else:
			raise Exception('Invalid value for payment parameter.')
		return purchase
	
	def rentals (self, limit = Query.DEFAULT_LIMIT, **namedParameters):
		return RentalsQuery(self.api, limit, **namedParameters)
	
	def _makeStreamPath (self, streamID, authmode, **namedParameters):
		return self.api.makeStreamPath(streamID, authmode, **namedParameters)
